"use strict";
// This code is copied directly from Vite source (it is not exported)
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.logServerError = void 0;
const chalk_1 = __importDefault(require("chalk"));
const strip_ansi_1 = __importDefault(require("strip-ansi"));
const splitRE = /\r?\n/;
function pad(source, n = 2) {
    const lines = source.split(splitRE);
    return lines.map((l) => ` `.repeat(n) + l).join(`\n`);
}
function logServerError(error, server) {
    server.ssrFixStacktrace(error);
    const msg = buildErrorMessage(error, [
        chalk_1.default.red(`Internal server error: ${error.message}`),
    ]);
    server.config.logger.error(msg, {
        clear: true,
        timestamp: true,
        error,
    });
    const sendError = () => server.ws.send({ type: 'error', err: prepareError(error) });
    // Wait until browser injects ViteErrorOverlay custom element
    setTimeout(sendError, 100);
    setTimeout(sendError, 250);
}
exports.logServerError = logServerError;
function prepareError(err) {
    return {
        message: strip_ansi_1.default(err.message),
        stack: strip_ansi_1.default(cleanStack(err.stack || '')),
        id: err.id,
        frame: strip_ansi_1.default(err.frame || ''),
        plugin: err.plugin,
        pluginCode: err.pluginCode,
        loc: err.loc,
    };
}
function buildErrorMessage(err, args = [], includeStack = true) {
    if (err.plugin)
        args.push(`  Plugin: ${chalk_1.default.magenta(err.plugin)}`);
    if (err.id)
        args.push(`  File: ${chalk_1.default.cyan(err.id)}`);
    if (err.frame)
        args.push(chalk_1.default.yellow(pad(err.frame)));
    if (includeStack && err.stack)
        args.push(pad(cleanStack(err.stack)));
    return args.join('\n');
}
function cleanStack(stack) {
    return stack
        .split(/\n/g)
        .filter((l) => /^\s*at/.test(l))
        .join('\n');
}
