"use strict";
const server_1 = require("./dev/server");
const pluginName = 'vite-ssr';
const entryServer = '/entry-server';
const entryClient = '/entry-client';
let lib;
function hasPlugin(plugins = [], name) {
    return !!plugins.flat().find((plugin) => (plugin.name || '').startsWith(name));
}
function hasDependency(dependency) {
    try {
        require.resolve(dependency);
        return true;
    }
    catch (error) {
        return false;
    }
}
function detectReactConfigFeatures(features = {}) {
    const external = [];
    let useApolloRenderer;
    // TODO use virtual modules for feature-detection
    if (hasDependency('@apollo/client/react/ssr')) {
        useApolloRenderer = features.reactApolloRenderer !== false;
    }
    else {
        external.push('@apollo/client');
    }
    return {
        ssr: { external },
        define: {
            __USE_APOLLO_RENDERER__: !!useApolloRenderer,
        },
    };
}
module.exports = function ViteSsrPlugin(options = {}) {
    return {
        name: pluginName,
        viteSsrOptions: options,
        config(config) {
            const plugins = config.plugins;
            const isVue = hasPlugin(plugins, 'vite:vue');
            const isReact = hasPlugin(plugins, 'vite:react') || hasPlugin(plugins, 'react-refresh');
            lib = isVue ? 'vue' : isReact ? 'react' : 'core';
            const detectedFeats = {
                ...(isReact && detectReactConfigFeatures(options.features)),
            };
            return {
                ...detectedFeats,
                ssr: {
                    ...detectedFeats.ssr,
                    noExternal: [pluginName],
                },
            };
        },
        configResolved: (config) => {
            const libPath = `/${lib}`;
            config.resolve.alias.push({
                find: /^vite-ssr(\/core|\/vue|\/react)?$/,
                replacement: pluginName + libPath + (config.build.ssr ? entryServer : entryClient),
                // @ts-ignore
                _viteSSR: true,
            });
            // @ts-ignore
            config.optimizeDeps = config.optimizeDeps || {};
            config.optimizeDeps.include = config.optimizeDeps.include || [];
            config.optimizeDeps.include.push(pluginName + libPath + entryClient, pluginName + libPath + entryServer);
        },
        async configureServer(server) {
            if (process.env.__DEV_MODE_SSR) {
                const handler = server_1.createSSRDevHandler(server, options);
                return () => server.middlewares.use(handler);
            }
        },
    };
};
