import { defer } from './defer';
const isRedirect = ({ status = 0 }) => status >= 300 && status < 400;
export function useSsrResponse() {
    const deferred = defer();
    let response = {};
    const writeResponse = (params) => {
        Object.assign(response, params);
        if (isRedirect(params)) {
            // Stop waiting for rendering when redirecting
            deferred.resolve(response);
        }
    };
    return {
        deferred,
        response,
        writeResponse,
        isRedirect: () => isRedirect(response),
        redirect: (location, status = 302) => writeResponse({ headers: { location }, status }),
    };
}
export function useClientRedirect(spaRedirect) {
    return {
        writeResponse: () => console.warn('[SSR] Do not call writeResponse in browser'),
        redirect: (location, status) => {
            if (location.startsWith('/')) {
                return spaRedirect(location);
            }
            else {
                window.location.href = location;
            }
        },
    };
}
